@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-#
-# $Id: disk_usage.py 1236 2011-12-13 19:00:35Z g.rodola $
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -11,14 +9,13 @@ List all mounted disk partitions a-la "df -h" command.
 """
 
 import sys
+import os
 import psutil
+from psutil._compat import print_
 
-def print_(s):
-    # python 2/3 compatibility layer
-    sys.stdout.write(s + '\n')
-    sys.stdout.flush()
 
 def bytes2human(n):
+    # http://code.activestate.com/recipes/578019
     # >>> bytes2human(10000)
     # '9.8K'
     # >>> bytes2human(100001221)
@@ -26,7 +23,7 @@ def bytes2human(n):
     symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
     prefix = {}
     for i, s in enumerate(symbols):
-        prefix[s] = 1 << (i+1)*10
+        prefix[s] = 1 << (i + 1) * 10
     for s in reversed(symbols):
         if n >= prefix[s]:
             value = float(n) / prefix[s]
@@ -38,14 +35,21 @@ def main():
     templ = "%-17s %8s %8s %8s %5s%% %9s  %s"
     print_(templ % ("Device", "Total", "Used", "Free", "Use ", "Type", "Mount"))
     for part in psutil.disk_partitions(all=False):
+        if os.name == 'nt':
+            if 'cdrom' in part.opts or part.fstype == '':
+                # skip cd-rom drives with no disk in it; they may raise
+                # ENOENT, pop-up a Windows GUI error for a non-ready
+                # partition or just hang.
+                continue
         usage = psutil.disk_usage(part.mountpoint)
-        print_(templ % (part.device,
-                        bytes2human(usage.total),
-                        bytes2human(usage.used),
-                        bytes2human(usage.free),
-                        int(usage.percent),
-                        part.fstype,
-                        part.mountpoint))
+        print_(templ % (
+            part.device,
+            bytes2human(usage.total),
+            bytes2human(usage.used),
+            bytes2human(usage.free),
+            int(usage.percent),
+            part.fstype,
+            part.mountpoint))
 
 if __name__ == '__main__':
     sys.exit(main())
