@@ -2,7 +2,7 @@
 #
 # $Id: iotop.py 1160 2011-10-14 18:50:36Z g.rodola@gmail.com $
 #
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -16,8 +16,8 @@ import sys
 import os
 if os.name != 'posix':
     sys.exit('platform not supported')
-import curses
 import atexit
+import curses
 import time
 
 import psutil
@@ -35,6 +35,7 @@ atexit.register(tear_down)
 curses.endwin()
 lineno = 0
 
+
 def print_line(line, highlight=False):
     """A thin wrapper around curses's addstr()."""
     global lineno
@@ -63,21 +64,22 @@ def bytes2human(n):
     symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
     prefix = {}
     for i, s in enumerate(symbols):
-        prefix[s] = 1 << (i+1)*10
+        prefix[s] = 1 << (i + 1) * 10
     for s in reversed(symbols):
         if n >= prefix[s]:
             value = float(n) / prefix[s]
             return '%.2f %s' % (value, s)
     return '%.2f B' % (n)
 
+
 def poll(interval):
     """Retrieve raw stats within an interval window."""
-    tot_before = psutil.network_io_counters()
-    pnic_before = psutil.network_io_counters(pernic=True)
+    tot_before = psutil.net_io_counters()
+    pnic_before = psutil.net_io_counters(pernic=True)
     # sleep some time
     time.sleep(interval)
-    tot_after = psutil.network_io_counters()
-    pnic_after = psutil.network_io_counters(pernic=True)
+    tot_after = psutil.net_io_counters()
+    pnic_after = psutil.net_io_counters(pernic=True)
     return (tot_before, tot_after, pnic_before, pnic_after)
 
 
@@ -86,19 +88,17 @@ def refresh_window(tot_before, tot_after, pnic_before, pnic_after):
     global lineno
 
     # totals
-    print_line("total bytes:           sent: %-10s   received: %s" \
-          % (bytes2human(tot_after.bytes_sent),
-             bytes2human(tot_after.bytes_recv))
+    print_line("total bytes:           sent: %-10s   received: %s" % (
+        bytes2human(tot_after.bytes_sent),
+        bytes2human(tot_after.bytes_recv))
     )
-    print_line("total packets:         sent: %-10s   received: %s" \
-          % (tot_after.packets_sent, tot_after.packets_recv)
-    )
-
+    print_line("total packets:         sent: %-10s   received: %s" % (
+        tot_after.packets_sent, tot_after.packets_recv))
 
     # per-network interface details: let's sort network interfaces so
     # that the ones which generated more traffic are shown first
     print_line("")
-    nic_names = pnic_after.keys()
+    nic_names = list(pnic_after.keys())
     nic_names.sort(key=lambda x: sum(pnic_after[x]), reverse=True)
     for name in nic_names:
         stats_before = pnic_before[name]
@@ -133,7 +133,7 @@ def refresh_window(tot_before, tot_after, pnic_before, pnic_after):
 def main():
     try:
         interval = 0
-        while 1:
+        while True:
             args = poll(interval)
             refresh_window(*args)
             interval = 1
