@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-#
-# $Id: _compat.py 1340 2012-06-09 13:42:21Z g.rodola $
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -12,27 +10,32 @@ __all__ = ["PY3", "int", "long", "xrange", "exec_", "callable",
            "namedtuple", "property", "defaultdict"]
 
 import sys
-
-
-# --- python 2/3 compatibility layer
-
-PY3 = sys.version_info >= (3,)
-
 try:
     import __builtin__
 except ImportError:
     import builtins as __builtin__  # py3
 
+PY3 = sys.version_info >= (3,)
+
+
 if PY3:
     int = int
     long = int
     xrange = range
+    unicode = str
     exec_ = getattr(__builtin__, "exec")
     print_ = getattr(__builtin__, "print")
+
+    def u(s):
+        return s
 else:
     int = int
     long = long
     xrange = xrange
+    unicode = unicode
+
+    def u(s):
+        return unicode(s, "unicode_escape")
 
     def exec_(code, globs=None, locs=None):
         if globs is None:
@@ -53,12 +56,9 @@ else:
 # removed in 3.0, reintroduced in 3.2
 try:
     callable = callable
-except Exception:
+except NameError:
     def callable(obj):
-        for klass in type(obj).__mro__:
-            if "__call__" in klass.__dict__:
-                return True
-        return False
+        return any("__call__" in klass.__dict__ for klass in type(obj).__mro__)
 
 
 # --- stdlib additions
@@ -87,22 +87,22 @@ except ImportError:
             names = list(field_names)
             seen = set()
             for i, name in enumerate(names):
-                if (not min(c.isalnum() or c=='_' for c in name) or _iskeyword(name)
+                if (not min(c.isalnum() or c == '_' for c in name) or _iskeyword(name)
                     or not name or name[0].isdigit() or name.startswith('_')
                     or name in seen):
-                        names[i] = '_%d' % i
+                    names[i] = '_%d' % i
                 seen.add(name)
             field_names = tuple(names)
         for name in (typename,) + field_names:
-            if not min(c.isalnum() or c=='_' for c in name):
-                raise ValueError('Type names and field names can only contain ' \
+            if not min(c.isalnum() or c == '_' for c in name):
+                raise ValueError('Type names and field names can only contain '
                                  'alphanumeric characters and underscores: %r'
                                  % name)
             if _iskeyword(name):
-                raise ValueError('Type names and field names cannot be a keyword: %r' \
+                raise ValueError('Type names and field names cannot be a keyword: %r'
                                  % name)
             if name[0].isdigit():
-                raise ValueError('Type names and field names cannot start with a ' \
+                raise ValueError('Type names and field names cannot start with a '
                                  'number: %r' % name)
         seen_names = set()
         for name in field_names:
@@ -151,8 +151,9 @@ except ImportError:
             sys.stdout.flush()
 
         # Execute the template string in a temporary namespace
-        namespace = dict(_itemgetter=_itemgetter, __name__='namedtuple_%s' % typename,
-                         _property=property, _tuple=tuple)
+        namespace = dict(
+            _itemgetter=_itemgetter, __name__='namedtuple_%s' % typename,
+            _property=property, _tuple=tuple)
         try:
             exec_(template, namespace)
         except SyntaxError:
@@ -166,7 +167,8 @@ except ImportError:
         # for example) or sys._getframe is not defined for arguments
         # greater than 0 (IronPython).
         try:
-            result.__module__ = _sys._getframe(1).f_globals.get('__name__', '__main__')
+            result.__module__ = _sys._getframe(
+                1).f_globals.get('__name__', '__main__')
         except (AttributeError, ValueError):
             pass
 
@@ -205,8 +207,8 @@ except ImportError:
     class defaultdict(dict):
 
         def __init__(self, default_factory=None, *a, **kw):
-            if (default_factory is not None and
-                not hasattr(default_factory, '__call__')):
+            if (default_factory is not None and \
+            not hasattr(default_factory, '__call__')):
                 raise TypeError('first argument must be callable')
             dict.__init__(self, *a, **kw)
             self.default_factory = default_factory
@@ -244,3 +246,27 @@ except ImportError:
         def __repr__(self):
             return 'defaultdict(%s, %s)' % (self.default_factory,
                                             dict.__repr__(self))
+
+
+# py 2.5 functools.wraps
+try:
+    from functools import wraps
+except ImportError:
+    def wraps(original):
+        def inner(fn):
+            # see functools.WRAPPER_ASSIGNMENTS
+            for attribute in ['__module__',
+                              '__name__',
+                              '__doc__'
+                              ]:
+                setattr(fn, attribute, getattr(original, attribute))
+            # see functools.WRAPPER_UPDATES
+            for attribute in ['__dict__',
+                              ]:
+                if hasattr(fn, attribute):
+                    getattr(fn, attribute).update(getattr(original, attribute))
+                else:
+                    setattr(fn, attribute,
+                            getattr(original, attribute).copy())
+            return fn
+        return inner
