@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-#
-# $Id: _psbsd.py 1386 2012-06-27 15:44:36Z g.rodola $
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -11,47 +9,106 @@
 import errno
 import os
 import sys
+import warnings
 
 import _psutil_bsd
 import _psutil_posix
+
 from psutil import _psposix
-from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
-from psutil._compat import namedtuple
 from psutil._common import *
+from psutil._compat import namedtuple, wraps
+from psutil._error import AccessDenied, NoSuchProcess, TimeoutExpired
 
 __extra__all__ = []
 
 # --- constants
 
-NUM_CPUS = _psutil_bsd.get_num_cpus()
-BOOT_TIME = _psutil_bsd.get_system_boot_time()
-_TERMINAL_MAP = _psposix._get_terminal_map()
+# Since these constants get determined at import time we do not want to
+# crash immediately; instead we'll set them to None and most likely
+# we'll crash later as they're used for determining process CPU stats
+# and creation_time
+try:
+    NUM_CPUS = _psutil_bsd.get_num_cpus()
+except Exception:
+    NUM_CPUS = None
+    warnings.warn("couldn't determine platform's NUM_CPUS", RuntimeWarning)
+try:
+    TOTAL_PHYMEM = _psutil_bsd.get_virtual_mem()[0]
+except Exception:
+    TOTAL_PHYMEM = None
+    warnings.warn("couldn't determine platform's TOTAL_PHYMEM", RuntimeWarning)
+try:
+    BOOT_TIME = _psutil_bsd.get_system_boot_time()
+except Exception:
+    BOOT_TIME = None
+    warnings.warn("couldn't determine platform's BOOT_TIME", RuntimeWarning)
+
+PROC_STATUSES = {
+    _psutil_bsd.SSTOP: STATUS_STOPPED,
+    _psutil_bsd.SSLEEP: STATUS_SLEEPING,
+    _psutil_bsd.SRUN: STATUS_RUNNING,
+    _psutil_bsd.SIDL: STATUS_IDLE,
+    _psutil_bsd.SWAIT: STATUS_WAITING,
+    _psutil_bsd.SLOCK: STATUS_LOCKED,
+    _psutil_bsd.SZOMB: STATUS_ZOMBIE,
+}
+
+TCP_STATUSES = {
+    _psutil_bsd.TCPS_ESTABLISHED: CONN_ESTABLISHED,
+    _psutil_bsd.TCPS_SYN_SENT: CONN_SYN_SENT,
+    _psutil_bsd.TCPS_SYN_RECEIVED: CONN_SYN_RECV,
+    _psutil_bsd.TCPS_FIN_WAIT_1: CONN_FIN_WAIT1,
+    _psutil_bsd.TCPS_FIN_WAIT_2: CONN_FIN_WAIT2,
+    _psutil_bsd.TCPS_TIME_WAIT: CONN_TIME_WAIT,
+    _psutil_bsd.TCPS_CLOSED: CONN_CLOSE,
+    _psutil_bsd.TCPS_CLOSE_WAIT: CONN_CLOSE_WAIT,
+    _psutil_bsd.TCPS_LAST_ACK: CONN_LAST_ACK,
+    _psutil_bsd.TCPS_LISTEN: CONN_LISTEN,
+    _psutil_bsd.TCPS_CLOSING: CONN_CLOSING,
+    _psutil_bsd.PSUTIL_CONN_NONE: CONN_NONE,
+}
+
+PAGESIZE = os.sysconf("SC_PAGE_SIZE")
+
+
+nt_virtmem_info = namedtuple('vmem', ' '.join([
+    # all platforms
+    'total', 'available', 'percent', 'used', 'free',
+    # FreeBSD specific
+    'active',
+    'inactive',
+    'buffers',
+    'cached',
+    'shared',
+    'wired']))
+
+def virtual_memory():
+    """System virtual memory as a namedutple."""
+    mem = _psutil_bsd.get_virtual_mem()
+    total, free, active, inactive, wired, cached, buffers, shared = mem
+    avail = inactive + cached + free
+    used = active + wired + cached
+    percent = usage_percent((total - avail), total, _round=1)
+    return nt_virtmem_info(total, avail, percent, used, free,
+                           active, inactive, buffers, cached, shared, wired)
+
+
+def swap_memory():
+    """System swap memory as (total, used, free, sin, sout) namedtuple."""
+    total, used, free, sin, sout = \
+        [x * PAGESIZE for x in _psutil_bsd.get_swap_mem()]
+    percent = usage_percent(used, total, _round=1)
+    return nt_swapmeminfo(total, used, free, percent, sin, sout)
+
+
 _cputimes_ntuple = namedtuple('cputimes', 'user nice system idle irq')
 
-# --- public functions
-
-def phymem_usage():
-    """Physical system memory as a (total, used, free) tuple."""
-    total = _psutil_bsd.get_total_phymem()
-    free =  _psutil_bsd.get_avail_phymem()
-    used = total - free
-    # XXX check out whether we have to do the same math we do on Linux
-    percent = usage_percent(used, total, _round=1)
-    return nt_sysmeminfo(total, used, free, percent)
-
-def virtmem_usage():
-    """Virtual system memory as a (total, used, free) tuple."""
-    total = _psutil_bsd.get_total_virtmem()
-    free =  _psutil_bsd.get_avail_virtmem()
-    used = total - free
-    percent = usage_percent(used, total, _round=1)
-    return nt_sysmeminfo(total, used, free, percent)
-
 def get_system_cpu_times():
     """Return system per-CPU times as a named tuple"""
     user, nice, system, idle, irq = _psutil_bsd.get_system_cpu_times()
     return _cputimes_ntuple(user, nice, system, idle, irq)
 
+
 def get_system_per_cpu_times():
     """Return system CPU times as a named tuple"""
     ret = []
@@ -76,8 +133,10 @@ if not hasattr(_psutil_bsd, "get_system_per_cpu_times"):
             raise NotImplementedError("supported only starting from FreeBSD 8")
         get_system_per_cpu_times.__called__ = True
         return [get_system_cpu_times]
+
 get_system_per_cpu_times.__called__ = False
 
+
 def disk_partitions(all=False):
     retlist = []
     partitions = _psutil_bsd.get_disk_partitions()
@@ -86,13 +145,13 @@ def disk_partitions(all=False):
         if device == 'none':
             device = ''
         if not all:
-            if not os.path.isabs(device) \
-            or not os.path.exists(device):
+            if not os.path.isabs(device) or not os.path.exists(device):
                 continue
         ntuple = nt_partition(device, mountpoint, fstype, opts)
         retlist.append(ntuple)
     return retlist
 
+
 def get_system_users():
     retlist = []
     rawlist = _psutil_bsd.get_system_users()
@@ -104,21 +163,25 @@ def get_system_users():
         retlist.append(nt)
     return retlist
 
+
 get_pid_list = _psutil_bsd.get_pid_list
 pid_exists = _psposix.pid_exists
 get_disk_usage = _psposix.get_disk_usage
-network_io_counters = _psutil_bsd.get_network_io_counters
+net_io_counters = _psutil_bsd.get_net_io_counters
 disk_io_counters = _psutil_bsd.get_disk_io_counters
+# not public; it's here because we need to test it from test_memory_leask.py
+get_num_cpus = _psutil_bsd.get_num_cpus()
+get_system_boot_time = _psutil_bsd.get_system_boot_time
 
 
-def wrap_exceptions(method):
-    """Call method(self, pid) into a try/except clause so that if an
-    OSError "No such process" exception is raised we assume the process
-    has died and raise psutil.NoSuchProcess instead.
+def wrap_exceptions(fun):
+    """Decorator which translates bare OSError exceptions into
+    NoSuchProcess and AccessDenied.
     """
+    @wraps(fun)
     def wrapper(self, *args, **kwargs):
         try:
-            return method(self, *args, **kwargs)
+            return fun(self, *args, **kwargs)
         except OSError:
             err = sys.exc_info()[1]
             if err.errno == errno.ESRCH:
@@ -128,16 +191,6 @@ def wrap_exceptions(method):
             raise
     return wrapper
 
-_status_map = {
-    _psutil_bsd.SSTOP : STATUS_STOPPED,
-    _psutil_bsd.SSLEEP : STATUS_SLEEPING,
-    _psutil_bsd.SRUN : STATUS_RUNNING,
-    _psutil_bsd.SIDL : STATUS_IDLE,
-    _psutil_bsd.SWAIT : STATUS_WAITING,
-    _psutil_bsd.SLOCK : STATUS_LOCKED,
-    _psutil_bsd.SZOMB : STATUS_ZOMBIE,
-}
-
 
 class Process(object):
     """Wrapper class around underlying C implementation."""
@@ -166,8 +219,9 @@ class Process(object):
     @wrap_exceptions
     def get_process_terminal(self):
         tty_nr = _psutil_bsd.get_process_tty_nr(self.pid)
+        tmap = _psposix._get_terminal_map()
         try:
-            return _TERMINAL_MAP[tty_nr]
+            return tmap[tty_nr]
         except KeyError:
             return None
 
@@ -200,15 +254,21 @@ class Process(object):
     @wrap_exceptions
     def get_cpu_times(self):
         """return a tuple containing process user/kernel time."""
-        user, system = _psutil_bsd.get_cpu_times(self.pid)
+        user, system = _psutil_bsd.get_process_cpu_times(self.pid)
         return nt_cputimes(user, system)
 
     @wrap_exceptions
     def get_memory_info(self):
         """Return a tuple with the process' RSS and VMS size."""
-        rss, vms = _psutil_bsd.get_memory_info(self.pid)
+        rss, vms = _psutil_bsd.get_process_memory_info(self.pid)[:2]
         return nt_meminfo(rss, vms)
 
+    _nt_ext_mem = namedtuple('meminfo', 'rss vms text data stack')
+
+    @wrap_exceptions
+    def get_ext_memory_info(self):
+        return self._nt_ext_mem(*_psutil_bsd.get_process_memory_info(self.pid))
+
     @wrap_exceptions
     def get_process_create_time(self):
         """Return the start time of the process as a number of seconds since
@@ -220,6 +280,10 @@ class Process(object):
         """Return the number of threads belonging to the process."""
         return _psutil_bsd.get_process_num_threads(self.pid)
 
+    @wrap_exceptions
+    def get_num_ctx_switches(self):
+        return nt_ctxsw(*_psutil_bsd.get_process_num_ctx_switches(self.pid))
+
     @wrap_exceptions
     def get_num_fds(self):
         """Return the number of file descriptors opened by this process."""
@@ -247,19 +311,22 @@ class Process(object):
             lsof = _psposix.LsofParser(self.pid, self._process_name)
             return lsof.get_process_open_files()
 
+    @wrap_exceptions
     def get_connections(self, kind='inet'):
-        """Return network connections opened by a process as a list of
-        namedtuples by parsing lsof output.
+        """Return etwork connections opened by a process as a list of
+        namedtuples.
         """
         if kind not in conn_tmap:
             raise ValueError("invalid %r kind argument; choose between %s"
                              % (kind, ', '.join([repr(x) for x in conn_tmap])))
         families, types = conn_tmap[kind]
+        rawlist = _psutil_bsd.get_process_connections(self.pid, families, types)
         ret = []
-        lsof = _psposix.LsofParser(self.pid, self._process_name)
-        for conn in lsof.get_process_connections():
-            if conn.family in families and conn.type in types:
-                ret.append(conn)
+        for item in rawlist:
+            fd, fam, type, laddr, raddr, status = item
+            status = TCP_STATUSES[status]
+            nt = nt_connection(fd, fam, type, laddr, raddr, status)
+            ret.append(nt)
         return ret
 
     @wrap_exceptions
@@ -280,19 +347,20 @@ class Process(object):
     @wrap_exceptions
     def get_process_status(self):
         code = _psutil_bsd.get_process_status(self.pid)
-        if code in _status_map:
-            return _status_map[code]
-        return constant(-1, "?")
+        if code in PROC_STATUSES:
+            return PROC_STATUSES[code]
+        # XXX is this legit? will we even ever get here?
+        return "?"
 
     @wrap_exceptions
     def get_process_io_counters(self):
         rc, wc, rb, wb = _psutil_bsd.get_process_io_counters(self.pid)
         return nt_io(rc, wc, rb, wb)
 
-    nt_mmap_grouped = namedtuple('mmap',
-        'path rss, private, ref_count, shadow_count')
-    nt_mmap_ext = namedtuple('mmap',
-        'addr, perms path rss, private, ref_count, shadow_count')
+    nt_mmap_grouped = namedtuple(
+        'mmap', 'path rss, private, ref_count, shadow_count')
+    nt_mmap_ext = namedtuple(
+        'mmap', 'addr, perms path rss, private, ref_count, shadow_count')
 
     @wrap_exceptions
     def get_memory_maps(self):
@@ -302,6 +370,7 @@ class Process(object):
     if not hasattr(_psutil_bsd, 'get_process_open_files'):
         def _not_implemented(self):
             raise NotImplementedError("supported only starting from FreeBSD 8")
+
         get_open_files = _not_implemented
         get_process_cwd = _not_implemented
         get_memory_maps = _not_implemented
