@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-#
-# $Id: _pslinux.py 1393 2012-06-28 15:26:27Z g.rodola $
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -10,43 +8,98 @@
 
 from __future__ import division
 
-import os
+import base64
 import errno
+import os
+import re
 import socket
 import struct
 import sys
-import base64
-import re
+import warnings
 
-import _psutil_posix
 import _psutil_linux
+import _psutil_posix
+
+from _psutil_linux import *  # needed for RLIMIT_* constants
 from psutil import _psposix
-from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
 from psutil._common import *
-from psutil._compat import PY3, xrange, long
+from psutil._compat import PY3, xrange, long, namedtuple, wraps
+from psutil._error import AccessDenied, NoSuchProcess, TimeoutExpired
+
 
 __extra__all__ = [
+    # io prio constants
     "IOPRIO_CLASS_NONE", "IOPRIO_CLASS_RT", "IOPRIO_CLASS_BE",
     "IOPRIO_CLASS_IDLE",
+    # connection status constants
+    "CONN_ESTABLISHED", "CONN_SYN_SENT", "CONN_SYN_RECV", "CONN_FIN_WAIT1",
+    "CONN_FIN_WAIT2", "CONN_TIME_WAIT", "CONN_CLOSE", "CONN_CLOSE_WAIT",
+    "CONN_LAST_ACK", "CONN_LISTEN", "CONN_CLOSING",
+    # other
     "phymem_buffers", "cached_phymem"]
 
+HAS_PRLIMIT = hasattr(_psutil_linux, "prlimit")
 
-def _get_boot_time():
-    """Return system boot time (epoch in seconds)"""
+# RLIMIT_* constants, not guaranteed to be present on all kernels
+if HAS_PRLIMIT:
+    for name in dir(_psutil_linux):
+        if name.startswith('RLIM'):
+            __extra__all__.append(name)
+
+# Number of clock ticks per second
+CLOCK_TICKS = os.sysconf("SC_CLK_TCK")
+PAGESIZE = os.sysconf("SC_PAGE_SIZE")
+
+# ioprio_* constants http://linux.die.net/man/2/ioprio_get
+IOPRIO_CLASS_NONE = 0
+IOPRIO_CLASS_RT = 1
+IOPRIO_CLASS_BE = 2
+IOPRIO_CLASS_IDLE = 3
+
+# taken from /fs/proc/array.c
+PROC_STATUSES = {
+    "R": STATUS_RUNNING,
+    "S": STATUS_SLEEPING,
+    "D": STATUS_DISK_SLEEP,
+    "T": STATUS_STOPPED,
+    "t": STATUS_TRACING_STOP,
+    "Z": STATUS_ZOMBIE,
+    "X": STATUS_DEAD,
+    "x": STATUS_DEAD,
+    "K": STATUS_WAKE_KILL,
+    "W": STATUS_WAKING
+}
+
+# http://students.mimuw.edu.pl/lxr/source/include/net/tcp_states.h
+TCP_STATUSES = {
+    "01": CONN_ESTABLISHED,
+    "02": CONN_SYN_SENT,
+    "03": CONN_SYN_RECV,
+    "04": CONN_FIN_WAIT1,
+    "05": CONN_FIN_WAIT2,
+    "06": CONN_TIME_WAIT,
+    "07": CONN_CLOSE,
+    "08": CONN_CLOSE_WAIT,
+    "09": CONN_LAST_ACK,
+    "0A": CONN_LISTEN,
+    "0B": CONN_CLOSING
+}
+
+
+def get_system_boot_time():
+    """Return the system boot time expressed in seconds since the epoch."""
     f = open('/proc/stat', 'r')
     try:
         for line in f:
             if line.startswith('btime'):
                 return float(line.strip().split()[1])
-        raise RuntimeError("line not found")
+        raise RuntimeError("line 'btime' not found")
     finally:
         f.close()
 
-def _get_num_cpus():
+
+def get_num_cpus():
     """Return the number of CPUs on the system"""
-    # we try to determine num CPUs by using different approaches.
-    # SC_NPROCESSORS_ONLN seems to be the safer and it is also
-    # used by multiprocessing module
     try:
         return os.sysconf("SC_NPROCESSORS_ONLN")
     except ValueError:
@@ -77,128 +130,182 @@ def _get_num_cpus():
                 num += 1
 
     if num == 0:
-        raise RuntimeError("can't determine number of CPUs")
+        raise RuntimeError("couldn't determine platform's NUM_CPUS")
     return num
 
 
-# Number of clock ticks per second
-_CLOCK_TICKS = os.sysconf(os.sysconf_names["SC_CLK_TCK"])
-_PAGESIZE = os.sysconf("SC_PAGE_SIZE")
-_TERMINAL_MAP = _psposix._get_terminal_map()
-BOOT_TIME = _get_boot_time()
-NUM_CPUS = _get_num_cpus()
-# ioprio_* constants http://linux.die.net/man/2/ioprio_get
-IOPRIO_CLASS_NONE = 0
-IOPRIO_CLASS_RT = 1
-IOPRIO_CLASS_BE = 2
-IOPRIO_CLASS_IDLE = 3
+# Since these constants get determined at import time we do not want to
+# crash immediately; instead we'll set them to None and most likely
+# we'll crash later as they're used for determining process CPU stats
+# and creation_time
+try:
+    BOOT_TIME = get_system_boot_time()
+except Exception:
+    BOOT_TIME = None
+    warnings.warn("couldn't determine platform's BOOT_TIME", RuntimeWarning)
+try:
+    NUM_CPUS = get_num_cpus()
+except Exception:
+    NUM_CPUS = None
+    warnings.warn("couldn't determine platform's NUM_CPUS", RuntimeWarning)
+try:
+    TOTAL_PHYMEM = _psutil_linux.get_sysinfo()[0]
+except Exception:
+    TOTAL_PHYMEM = None
+    warnings.warn("couldn't determine platform's TOTAL_PHYMEM", RuntimeWarning)
 
-# http://students.mimuw.edu.pl/lxr/source/include/net/tcp_states.h
-_TCP_STATES_TABLE = {"01" : "ESTABLISHED",
-                     "02" : "SYN_SENT",
-                     "03" : "SYN_RECV",
-                     "04" : "FIN_WAIT1",
-                     "05" : "FIN_WAIT2",
-                     "06" : "TIME_WAIT",
-                     "07" : "CLOSE",
-                     "08" : "CLOSE_WAIT",
-                     "09" : "LAST_ACK",
-                     "0A" : "LISTEN",
-                     "0B" : "CLOSING"
-                     }
 
-# --- system memory functions
+# --- system memory
 
-def cached_phymem():
-    """Return the amount of cached memory on the system, in bytes.
-    This reflects the "cached" column of free command line utility.
-    """
+nt_virtmem_info = namedtuple('vmem', ' '.join([
+    # all platforms
+    'total', 'available', 'percent', 'used', 'free',
+    # linux specific
+    'active',
+    'inactive',
+    'buffers',
+    'cached']))
+
+def virtual_memory():
+    total, free, buffers, shared, _, _ = _psutil_linux.get_sysinfo()
+    cached = active = inactive = None
     f = open('/proc/meminfo', 'r')
     try:
         for line in f:
             if line.startswith('Cached:'):
-                return int(line.split()[1]) * 1024
-        raise RuntimeError("line not found")
-    finally:
-        f.close()
-
-def phymem_buffers():
-    """Return the amount of physical memory buffers used by the
-    kernel in bytes.
-    This reflects the "buffers" column of free command line utility.
-    """
-    total, free, buffers = _psutil_linux.get_physmem()
-    return buffers
-
-def phymem_usage():
-    """Return physical memory usage statistics as a namedutple including
-    tota, used, free and percent usage.
-    """
-    # total, used and free values are matched against free cmdline utility
-    # the percentage matches top/htop and gnome-system-monitor
-    total, free, buffers = _psutil_linux.get_physmem()
-    cached = cached_phymem()
-    used = total - free
-    percent = usage_percent(total - (free + buffers + cached), total, _round=1)
-    return nt_sysmeminfo(total, used, free, percent)
-
-
-def virtmem_usage():
-    f = open('/proc/meminfo', 'r')
-    try:
-        total = free = None
-        for line in f:
-            if line.startswith('SwapTotal:'):
-                total = int(line.split()[1]) * 1024
-            elif line.startswith('SwapFree:'):
-                free = int(line.split()[1]) * 1024
-            if total is not None and free is not None:
+                cached = int(line.split()[1]) * 1024
+            elif line.startswith('Active:'):
+                active = int(line.split()[1]) * 1024
+            elif line.startswith('Inactive:'):
+                inactive = int(line.split()[1]) * 1024
+            if cached is not None \
+            and active is not None \
+            and inactive is not None:
                 break
-        assert total is not None and free is not None
-        used = total - free
-        percent = usage_percent(used, total, _round=1)
-        return nt_sysmeminfo(total, used, free, percent)
+        else:
+            # we might get here when dealing with exotic Linux flavors, see:
+            # http://code.google.com/p/psutil/issues/detail?id=313
+            msg = "'cached', 'active' and 'inactive' memory stats couldn't " \
+                  "be determined and were set to 0"
+            warnings.warn(msg, RuntimeWarning)
+            cached = active = inactive = 0
     finally:
         f.close()
+    avail = free + buffers + cached
+    used = total - free
+    percent = usage_percent((total - avail), total, _round=1)
+    return nt_virtmem_info(total, avail, percent, used, free,
+                           active, inactive, buffers, cached)
 
 
-# --- system CPU functions
+def swap_memory():
+    _, _, _, _, total, free = _psutil_linux.get_sysinfo()
+    used = total - free
+    percent = usage_percent(used, total, _round=1)
+    # get pgin/pgouts
+    f = open("/proc/vmstat", "r")
+    sin = sout = None
+    try:
+        for line in f:
+            # values are expressed in 4 kilo bytes, we want bytes instead
+            if line.startswith('pswpin'):
+                sin = int(line.split(' ')[1]) * 4 * 1024
+            elif line.startswith('pswpout'):
+                sout = int(line.split(' ')[1]) * 4 * 1024
+            if sin is not None and sout is not None:
+                break
+        else:
+            # we might get here when dealing with exotic Linux flavors, see:
+            # http://code.google.com/p/psutil/issues/detail?id=313
+            msg = "'sin' and 'sout' swap memory stats couldn't " \
+                  "be determined and were set to 0"
+            warnings.warn(msg, RuntimeWarning)
+            sin = sout = 0
+    finally:
+        f.close()
+    return nt_swapmeminfo(total, used, free, percent, sin, sout)
+
+
+@deprecated('psutil.virtual_memory().cached')
+def cached_phymem():
+    return virtual_memory().cached
+
+
+@deprecated('psutil.virtual_memory().buffers')
+def phymem_buffers():
+    return virtual_memory().buffers
+
+
+# --- CPU
+
+@memoize
+def _get_cputimes_ntuple():
+    """ Return a (nt, rindex) tuple depending on the CPU times available
+    on this Linux kernel version which may be:
+    user, nice, system, idle, iowait, irq, softirq [steal, [guest, [guest_nice]]]
+    """
+    f = open('/proc/stat', 'r')
+    try:
+        values = f.readline().split()[1:]
+    finally:
+        f.close()
+    fields = ['user', 'nice', 'system', 'idle', 'iowait', 'irq', 'softirq']
+    rindex = 8
+    vlen = len(values)
+    if vlen >= 8:
+        # Linux >= 2.6.11
+        fields.append('steal')
+        rindex += 1
+    if vlen >= 9:
+        # Linux >= 2.6.24
+        fields.append('guest')
+        rindex += 1
+    if vlen >= 10:
+        # Linux >= 3.2.0
+        fields.append('guest_nice')
+        rindex += 1
+    return (namedtuple('cputimes', ' '.join(fields)), rindex)
+
 
 def get_system_cpu_times():
-    """Return a named tuple representing the following CPU times:
-    user, nice, system, idle, iowait, irq, softirq.
+    """Return a named tuple representing the following system-wide
+    CPU times:
+    user, nice, system, idle, iowait, irq, softirq [steal, [guest, [guest_nice]]]
+    Last 3 fields may not be available on all Linux kernel versions.
     """
     f = open('/proc/stat', 'r')
     try:
         values = f.readline().split()
     finally:
         f.close()
+    nt, rindex = _get_cputimes_ntuple()
+    fields = values[1:rindex]
+    fields = [float(x) / CLOCK_TICKS for x in fields]
+    return nt(*fields)
 
-    values = values[1:8]
-    values = tuple([float(x) / _CLOCK_TICKS for x in values])
-    return nt_sys_cputimes(*values[:7])
 
 def get_system_per_cpu_times():
     """Return a list of namedtuple representing the CPU times
     for every CPU available on the system.
     """
+    nt, rindex = _get_cputimes_ntuple()
     cpus = []
     f = open('/proc/stat', 'r')
-    # get rid of the first line who refers to system wide CPU stats
     try:
+        # get rid of the first line which refers to system wide CPU stats
         f.readline()
-        for line in f.readlines():
+        for line in f:
             if line.startswith('cpu'):
-                values = line.split()[1:8]
-                values = tuple([float(x) / _CLOCK_TICKS for x in values])
-                entry = nt_sys_cputimes(*values[:7])
+                fields = line.split()[1:rindex]
+                fields = [float(x) / CLOCK_TICKS for x in fields]
+                entry = nt(*fields)
                 cpus.append(entry)
         return cpus
     finally:
         f.close()
 
 
-# --- system disk functions
+# --- disks
 
 def disk_partitions(all=False):
     """Return mounted disk partitions as a list of nameduples"""
@@ -227,7 +334,7 @@ def disk_partitions(all=False):
 get_disk_usage = _psposix.get_disk_usage
 
 
-# --- other sysetm functions
+# --- other system functions
 
 def get_system_users():
     """Return currently connected users as a list of namedtuples."""
@@ -235,7 +342,7 @@ def get_system_users():
     rawlist = _psutil_linux.get_system_users()
     for item in rawlist:
         user, tty, hostname, tstamp, user_process = item
-        # XXX the underlying C function includes entries about
+        # note: the underlying C function includes entries about
         # system boot, run level and others.  We might want
         # to use them in the future.
         if not user_process:
@@ -246,18 +353,23 @@ def get_system_users():
         retlist.append(nt)
     return retlist
 
-# --- process functions
+
+# --- processes
 
 def get_pid_list():
     """Returns a list of PIDs currently running on the system."""
     pids = [int(x) for x in os.listdir('/proc') if x.isdigit()]
     return pids
 
+
 def pid_exists(pid):
     """Check For the existence of a unix pid."""
     return _psposix.pid_exists(pid)
 
-def network_io_counters():
+
+# --- network
+
+def net_io_counters():
     """Return network I/O statistics for every network interface
     installed on the system as a dict of raw tuples.
     """
@@ -269,17 +381,25 @@ def network_io_counters():
 
     retdict = {}
     for line in lines[2:]:
-        colon = line.find(':')
-        assert colon > 0, line
+        colon = line.rfind(':')
+        assert colon > 0, repr(line)
         name = line[:colon].strip()
-        fields = line[colon+1:].strip().split()
+        fields = line[colon + 1:].strip().split()
         bytes_recv = int(fields[0])
         packets_recv = int(fields[1])
+        errin = int(fields[2])
+        dropin = int(fields[3])
         bytes_sent = int(fields[8])
         packets_sent = int(fields[9])
-        retdict[name] = (bytes_sent, bytes_recv, packets_sent, packets_recv)
+        errout = int(fields[10])
+        dropout = int(fields[11])
+        retdict[name] = (bytes_sent, bytes_recv, packets_sent, packets_recv,
+                         errin, errout, dropin, dropout)
     return retdict
 
+
+# --- disks
+
 def disk_io_counters():
     """Return disk I/O statistics for every disk installed on the
     system as a dict of raw tuples.
@@ -296,10 +416,19 @@ def disk_io_counters():
         lines = f.readlines()[2:]
     finally:
         f.close()
-    for line in lines:
+    for line in reversed(lines):
         _, _, _, name = line.split()
         if name[-1].isdigit():
+            # we're dealing with a partition (e.g. 'sda1'); 'sda' will
+            # also be around but we want to omit it
             partitions.append(name)
+        else:
+            if not partitions or not partitions[-1].startswith(name):
+                # we're dealing with a disk entity for which no
+                # partitions have been defined (e.g. 'sda' but
+                # 'sda1' was not around), see:
+                # http://code.google.com/p/psutil/issues/detail?id=338
+                partitions.append(name)
     #
     retdict = {}
     f = open("/proc/diskstats", "r")
@@ -308,6 +437,7 @@ def disk_io_counters():
     finally:
         f.close()
     for line in lines:
+        # http://www.mjmwired.net/kernel/Documentation/iostats.txt
         _, _, name, reads, _, rbytes, rtime, writes, _, wbytes, wtime = \
             line.split()[:11]
         if name in partitions:
@@ -315,35 +445,22 @@ def disk_io_counters():
             wbytes = int(wbytes) * SECTOR_SIZE
             reads = int(reads)
             writes = int(writes)
-            # TODO: times are expressed in milliseconds while OSX/BSD has
-            # these expressed in nanoseconds; figure this out.
             rtime = int(rtime)
             wtime = int(wtime)
             retdict[name] = (reads, writes, rbytes, wbytes, rtime, wtime)
     return retdict
 
 
-# taken from /fs/proc/array.c
-_status_map = {"R" : STATUS_RUNNING,
-               "S" : STATUS_SLEEPING,
-               "D" : STATUS_DISK_SLEEP,
-               "T" : STATUS_STOPPED,
-               "t" : STATUS_TRACING_STOP,
-               "Z" : STATUS_ZOMBIE,
-               "X" : STATUS_DEAD,
-               "x" : STATUS_DEAD,
-               "K" : STATUS_WAKE_KILL,
-               "W" : STATUS_WAKING}
-
 # --- decorators
 
-def wrap_exceptions(callable):
-    """Call callable into a try/except clause and translate ENOENT,
-    EACCES and EPERM in NoSuchProcess or AccessDenied exceptions.
+def wrap_exceptions(fun):
+    """Decorator which translates bare OSError and IOError exceptions
+    into NoSuchProcess and AccessDenied.
     """
+    @wraps(fun)
     def wrapper(self, *args, **kwargs):
         try:
-            return callable(self, *args, **kwargs)
+            return fun(self, *args, **kwargs)
         except EnvironmentError:
             # ENOENT (no such file or directory) gets raised on open().
             # ESRCH (no such process) can get raised on read() if
@@ -363,8 +480,6 @@ class Process(object):
     __slots__ = ["pid", "_process_name"]
 
     def __init__(self, pid):
-        if not isinstance(pid, int):
-            raise TypeError('pid must be an integer')
         self.pid = pid
         self._process_name = None
 
@@ -379,8 +494,6 @@ class Process(object):
         return name
 
     def get_process_exe(self):
-        if self.pid in (0, 2):
-            raise AccessDenied(self.pid, self._process_name)
         try:
             exe = os.readlink("/proc/%s/exe" % self.pid)
         except (OSError, IOError):
@@ -402,13 +515,12 @@ class Process(object):
         # problems when used with other fs-related functions (os.*,
         # open(), ...)
         exe = exe.replace('\x00', '')
-        # It seems symlinks can point to a deleted/invalid location
-        # (this usually  happens with "pulseaudio" process).
-        # However, if we had permissions to execute readlink() it's
-        # likely that we'll be able to figure out exe from argv[0]
-        # later on.
-        if exe.endswith(" (deleted)") and not os.path.isfile(exe):
-            return ""
+        # Certain names have ' (deleted)' appended. Usually this is
+        # bogus as the file actually exists. Either way that's not
+        # important as we don't want to discriminate executables which
+        # have been deleted.
+        if exe.endswith(" (deleted)") and not os.path.exists(exe):
+            exe = exe[:-10]
         return exe
 
     @wrap_exceptions
@@ -422,36 +534,44 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_terminal(self):
+        tmap = _psposix._get_terminal_map()
         f = open("/proc/%s/stat" % self.pid)
         try:
             tty_nr = int(f.read().split(' ')[6])
         finally:
             f.close()
         try:
-            return _TERMINAL_MAP[tty_nr]
+            return tmap[tty_nr]
         except KeyError:
             return None
 
-    @wrap_exceptions
-    def get_process_io_counters(self):
-        f = open("/proc/%s/io" % self.pid)
-        try:
-            for line in f:
-                if line.startswith("rchar"):
-                    read_count = int(line.split()[1])
-                elif line.startswith("wchar"):
-                    write_count = int(line.split()[1])
-                elif line.startswith("read_bytes"):
-                    read_bytes = int(line.split()[1])
-                elif line.startswith("write_bytes"):
-                    write_bytes = int(line.split()[1])
-            return nt_io(read_count, write_count, read_bytes, write_bytes)
-        finally:
-            f.close()
-
-    if not os.path.exists('/proc/%s/io' % os.getpid()):
+    if os.path.exists('/proc/%s/io' % os.getpid()):
+        @wrap_exceptions
         def get_process_io_counters(self):
-            raise NotImplementedError('/proc/PID/io is not available')
+            fname = "/proc/%s/io" % self.pid
+            f = open(fname)
+            try:
+                rcount = wcount = rbytes = wbytes = None
+                for line in f:
+                    if rcount is None and line.startswith("syscr"):
+                        rcount = int(line.split()[1])
+                    elif wcount is None and line.startswith("syscw"):
+                        wcount = int(line.split()[1])
+                    elif rbytes is None and line.startswith("read_bytes"):
+                        rbytes = int(line.split()[1])
+                    elif wbytes is None and line.startswith("write_bytes"):
+                        wbytes = int(line.split()[1])
+                for x in (rcount, wcount, rbytes, wbytes):
+                    if x is None:
+                        raise NotImplementedError(
+                            "couldn't read all necessary info from %r" % fname)
+                return nt_io(rcount, wcount, rbytes, wbytes)
+            finally:
+                f.close()
+    else:
+        def get_process_io_counters(self):
+            raise NotImplementedError("couldn't find /proc/%s/io (kernel "
+                                      "too old?)" % self.pid)
 
     @wrap_exceptions
     def get_cpu_times(self):
@@ -463,8 +583,8 @@ class Process(object):
         # ignore the first two values ("pid (exe)")
         st = st[st.find(')') + 2:]
         values = st.split(' ')
-        utime = float(values[11]) / _CLOCK_TICKS
-        stime = float(values[12]) / _CLOCK_TICKS
+        utime = float(values[11]) / CLOCK_TICKS
+        stime = float(values[12]) / CLOCK_TICKS
         return nt_cputimes(utime, stime)
 
     @wrap_exceptions
@@ -488,7 +608,7 @@ class Process(object):
         # unit is jiffies (clock ticks).
         # We first divide it for clock ticks and then add uptime returning
         # seconds since the epoch, in UTC.
-        starttime = (float(values[19]) / _CLOCK_TICKS) + BOOT_TIME
+        starttime = (float(values[19]) / CLOCK_TICKS) + BOOT_TIME
         return starttime
 
     @wrap_exceptions
@@ -496,14 +616,37 @@ class Process(object):
         f = open("/proc/%s/statm" % self.pid)
         try:
             vms, rss = f.readline().split()[:2]
-            return nt_meminfo(int(rss) * _PAGESIZE,
-                              int(vms) * _PAGESIZE)
+            return nt_meminfo(int(rss) * PAGESIZE,
+                              int(vms) * PAGESIZE)
         finally:
             f.close()
 
+    _nt_ext_mem = namedtuple('meminfo', 'rss vms shared text lib data dirty')
+
+    @wrap_exceptions
+    def get_ext_memory_info(self):
+        #  ============================================================
+        # | FIELD  | DESCRIPTION                         | AKA  | TOP  |
+        #  ============================================================
+        # | rss    | resident set size                   |      | RES  |
+        # | vms    | total program size                  | size | VIRT |
+        # | shared | shared pages (from shared mappings) |      | SHR  |
+        # | text   | text ('code')                       | trs  | CODE |
+        # | lib    | library (unused in Linux 2.6)       | lrs  |      |
+        # | data   | data + stack                        | drs  | DATA |
+        # | dirty  | dirty pages (unused in Linux 2.6)   | dt   |      |
+        #  ============================================================
+        f = open("/proc/%s/statm" % self.pid)
+        try:
+            vms, rss, shared, text, lib, data, dirty = \
+                [int(x) * PAGESIZE for x in f.readline().split()[:7]]
+        finally:
+            f.close()
+        return self._nt_ext_mem(rss, vms, shared, text, lib, data, dirty)
+
     _mmap_base_fields = ['path', 'rss', 'size', 'pss', 'shared_clean',
                          'shared_dirty', 'private_clean', 'private_dirty',
-                         'referenced', 'anonymous', 'swap',]
+                         'referenced', 'anonymous', 'swap', ]
     nt_mmap_grouped = namedtuple('mmap', ' '.join(_mmap_base_fields))
     nt_mmap_ext = namedtuple('mmap', 'addr perms ' + ' '.join(_mmap_base_fields))
 
@@ -522,11 +665,20 @@ class Process(object):
                 data = {}
                 for line in f:
                     fields = line.split(None, 5)
-                    if len(fields) >= 5:
+                    if not fields[0].endswith(':'):
+                        # new block section
                         yield (current_block.pop(), data)
                         current_block.append(line)
                     else:
-                        data[fields[0]] = int(fields[1]) * 1024
+                        try:
+                            data[fields[0]] = int(fields[1]) * 1024
+                        except ValueError:
+                            if fields[0].startswith('VmFlags:'):
+                                # see issue #369
+                                continue
+                            else:
+                                raise ValueError("don't know how to interpret"
+                                                 " line %r" % line)
                 yield (current_block.pop(), data)
 
             if first_line:  # smaps file can be empty
@@ -542,13 +694,15 @@ class Process(object):
                         path = path.strip()
                     yield (addr, perms, path,
                            data['Rss:'],
-                           data['Size:'],
+                           data.get('Size:', 0),
                            data.get('Pss:', 0),
-                           data['Shared_Clean:'], data['Shared_Clean:'],
-                           data['Private_Clean:'], data['Private_Dirty:'],
-                           data['Referenced:'],
-                           data['Anonymous:'],
-                           data['Swap:'])
+                           data.get('Shared_Clean:', 0),
+                           data.get('Shared_Dirty:', 0),
+                           data.get('Private_Clean:', 0),
+                           data.get('Private_Dirty:', 0),
+                           data.get('Referenced:', 0),
+                           data.get('Anonymous:', 0),
+                           data.get('Swap:', 0))
             f.close()
         except EnvironmentError:
             # XXX - Can't use wrap_exceptions decorator as we're
@@ -566,12 +720,12 @@ class Process(object):
             if f is not None:
                 f.close()
             raise
+        f.close()
 
     if not os.path.exists('/proc/%s/smaps' % os.getpid()):
-        def get_shared_libs(self, ext):
-            msg = "this Linux version does not support /proc/PID/smaps " \
-                  "(kernel < 2.6.14 or CONFIG_MMU kernel configuration " \
-                  "option is not enabled)"
+        def get_memory_maps(self, ext):
+            msg = "couldn't find /proc/%s/smaps; kernel < 2.6.14 or CONFIG_MMU " \
+                  "kernel configuration option is not enabled" % self.pid
             raise NotImplementedError(msg)
 
     @wrap_exceptions
@@ -582,6 +736,25 @@ class Process(object):
         path = os.readlink("/proc/%s/cwd" % self.pid)
         return path.replace('\x00', '')
 
+    @wrap_exceptions
+    def get_num_ctx_switches(self):
+        vol = unvol = None
+        f = open("/proc/%s/status" % self.pid)
+        try:
+            for line in f:
+                if line.startswith("voluntary_ctxt_switches"):
+                    vol = int(line.split()[1])
+                elif line.startswith("nonvoluntary_ctxt_switches"):
+                    unvol = int(line.split()[1])
+                if vol is not None and unvol is not None:
+                    return nt_ctxsw(vol, unvol)
+            raise NotImplementedError(
+                "'voluntary_ctxt_switches' and 'nonvoluntary_ctxt_switches'"
+                "fields were not found in /proc/%s/status; the kernel is "
+                "probably older than 2.6.23" % self.pid)
+        finally:
+            f.close()
+
     @wrap_exceptions
     def get_process_num_threads(self):
         f = open("/proc/%s/status" % self.pid)
@@ -589,7 +762,7 @@ class Process(object):
             for line in f:
                 if line.startswith("Threads:"):
                     return int(line.split()[1])
-            raise RuntimeError("line not found")
+            raise NotImplementedError("line not found")
         finally:
             f.close()
 
@@ -617,8 +790,8 @@ class Process(object):
             # ignore the first two values ("pid (exe)")
             st = st[st.find(')') + 2:]
             values = st.split(' ')
-            utime = float(values[11]) / _CLOCK_TICKS
-            stime = float(values[12]) / _CLOCK_TICKS
+            utime = float(values[11]) / CLOCK_TICKS
+            stime = float(values[12]) / CLOCK_TICKS
             ntuple = nt_thread(int(thread_id), utime, stime)
             retlist.append(ntuple)
         if hit_enoent:
@@ -629,10 +802,10 @@ class Process(object):
     @wrap_exceptions
     def get_process_nice(self):
         #f = open('/proc/%s/stat' % self.pid, 'r')
-        #try:
+        # try:
         #   data = f.read()
         #   return int(data.split()[18])
-        #finally:
+        # finally:
         #   f.close()
 
         # Use C implementation
@@ -657,7 +830,7 @@ class Process(object):
             for b in l:
                 if not isinstance(b, (int, long)) or b < 0:
                     raise ValueError("invalid argument %r" % b)
-                out |= 2**b
+                out |= 2 ** b
             return out
 
         bitmask = to_bitmask(value)
@@ -666,10 +839,11 @@ class Process(object):
         except OSError:
             err = sys.exc_info()[1]
             if err.errno == errno.EINVAL:
-                allcpus = list(range(len(get_system_per_cpu_times())))
+                allcpus = tuple(range(len(get_system_per_cpu_times())))
                 for cpu in value:
                     if cpu not in allcpus:
-                        raise ValueError("invalid CPU %i" % cpu)
+                        raise ValueError("invalid CPU #%i (choose between %s)"
+                                         % (cpu, allcpus))
             raise
 
     # only starting from kernel 2.6.13
@@ -697,9 +871,24 @@ class Process(object):
             else:
                 value = 0
             if not 0 <= value <= 8:
-                raise ValueError("value argument range expected is between 0 and 8")
+                raise ValueError(
+                    "value argument range expected is between 0 and 8")
             return _psutil_linux.ioprio_set(self.pid, ioclass, value)
 
+    if HAS_PRLIMIT:
+        @wrap_exceptions
+        def process_rlimit(self, resource, limits=None):
+            if limits is None:
+                # get
+                return _psutil_linux.prlimit(self.pid, resource)
+            else:
+                # set
+                if len(limits) != 2:
+                    raise ValueError(
+                        "second argument must be a (soft, hard) tuple")
+                soft, hard = limits
+                _psutil_linux.prlimit(self.pid, resource, soft, hard)
+
     @wrap_exceptions
     def get_process_status(self):
         f = open("/proc/%s/status" % self.pid)
@@ -707,9 +896,9 @@ class Process(object):
             for line in f:
                 if line.startswith("State:"):
                     letter = line.split()[1]
-                    if letter in _status_map:
-                        return _status_map[letter]
-                    return constant(-1, '?')
+                    # XXX is '?' legit? (we're not supposed to return
+                    # it anyway)
+                    return PROC_STATUSES.get(letter, '?')
         finally:
             f.close()
 
@@ -735,7 +924,7 @@ class Process(object):
                     # to tell whether it's a regular file or not,
                     # so we skip it. A regular file is always supposed
                     # to be absolutized though.
-                    if file.startswith('/') and os.path.isfile(file):
+                    if file.startswith('/') and isfile_strict(file):
                         ntuple = nt_openfile(file, int(fd))
                         retlist.append(ntuple)
         if hit_enoent:
@@ -761,6 +950,9 @@ class Process(object):
         udp6            UDP over IPv6
         all             the sum of all the possible families and protocols
         """
+        # Note: in case of UNIX sockets we're only able to determine the
+        # local bound path while the remote endpoint is not retrievable:
+        # http://goo.gl/R3GHM
         inodes = {}
         # os.listdir() is gonna raise a lot of access denied
         # exceptions in case of unprivileged user; that's fine:
@@ -796,38 +988,54 @@ class Process(object):
                     # IPv4 / IPv6
                     if family in (socket.AF_INET, socket.AF_INET6):
                         _, laddr, raddr, status, _, _, _, _, _, inode = \
-                                                                line.split()[:10]
+                            line.split()[:10]
                         if inode in inodes:
                             laddr = self._decode_address(laddr, family)
                             raddr = self._decode_address(raddr, family)
                             if type_ == socket.SOCK_STREAM:
-                                status = _TCP_STATES_TABLE[status]
+                                status = TCP_STATUSES[status]
                             else:
-                                status = ""
+                                status = CONN_NONE
                             fd = int(inodes[inode])
                             conn = nt_connection(fd, family, type_, laddr,
                                                  raddr, status)
                             retlist.append(conn)
+                    elif family == socket.AF_UNIX:
+                        tokens = line.split()
+                        _, _, _, _, type_, _, inode = tokens[0:7]
+                        if inode in inodes:
+
+                            if len(tokens) == 8:
+                                path = tokens[-1]
+                            else:
+                                path = ""
+                            fd = int(inodes[inode])
+                            type_ = int(type_)
+                            conn = nt_connection(fd, family, type_, path,
+                                                 None, CONN_NONE)
+                            retlist.append(conn)
                     else:
                         raise ValueError(family)
                 return retlist
             finally:
                 f.close()
 
-        tcp4 = ("tcp" , socket.AF_INET , socket.SOCK_STREAM)
+        tcp4 = ("tcp", socket.AF_INET, socket.SOCK_STREAM)
         tcp6 = ("tcp6", socket.AF_INET6, socket.SOCK_STREAM)
-        udp4 = ("udp" , socket.AF_INET , socket.SOCK_DGRAM)
+        udp4 = ("udp", socket.AF_INET, socket.SOCK_DGRAM)
         udp6 = ("udp6", socket.AF_INET6, socket.SOCK_DGRAM)
+        unix = ("unix", socket.AF_UNIX, None)
 
         tmap = {
-            "all"  : (tcp4, tcp6, udp4, udp6),
-            "tcp"  : (tcp4, tcp6),
-            "tcp4" : (tcp4,),
-            "tcp6" : (tcp6,),
-            "udp"  : (udp4, udp6),
-            "udp4" : (udp4,),
-            "udp6" : (udp6,),
-            "inet" : (tcp4, tcp6, udp4, udp6),
+            "all": (tcp4, tcp6, udp4, udp6, unix),
+            "tcp": (tcp4, tcp6),
+            "tcp4": (tcp4,),
+            "tcp6": (tcp6,),
+            "udp": (udp4, udp6),
+            "udp4": (udp4,),
+            "udp6": (udp6,),
+            "unix": (unix,),
+            "inet": (tcp4, tcp6, udp4, udp6),
             "inet4": (tcp4, udp4),
             "inet6": (tcp6, udp6),
         }
@@ -841,16 +1049,9 @@ class Process(object):
         os.stat('/proc/%s' % self.pid)
         return ret
 
-
-#    --- lsof implementation
-#
-#    def get_connections(self):
-#        lsof = _psposix.LsofParser(self.pid, self._process_name)
-#        return lsof.get_process_connections()
-
     @wrap_exceptions
     def get_num_fds(self):
-       return len(os.listdir("/proc/%s/fd" % self.pid))
+        return len(os.listdir("/proc/%s/fd" % self.pid))
 
     @wrap_exceptions
     def get_process_ppid(self):
@@ -860,7 +1061,7 @@ class Process(object):
                 if line.startswith("PPid:"):
                     # PPid: nnnn
                     return int(line.split()[1])
-            raise RuntimeError("line not found")
+            raise NotImplementedError("line not found")
         finally:
             f.close()
 
@@ -872,7 +1073,7 @@ class Process(object):
                 if line.startswith('Uid:'):
                     _, real, effective, saved, fs = line.split()
                     return nt_uids(int(real), int(effective), int(saved))
-            raise RuntimeError("line not found")
+            raise NotImplementedError("line not found")
         finally:
             f.close()
 
@@ -884,7 +1085,7 @@ class Process(object):
                 if line.startswith('Gid:'):
                     _, real, effective, saved, fs = line.split()
                     return nt_gids(int(real), int(effective), int(saved))
-            raise RuntimeError("line not found")
+            raise NotImplementedError("line not found")
         finally:
             f.close()
 
@@ -921,15 +1122,17 @@ class Process(object):
                 ip = socket.inet_ntop(family, base64.b16decode(ip))
         else:  # IPv6
             # old version - let's keep it, just in case...
-            #ip = ip.decode('hex')
-            #return socket.inet_ntop(socket.AF_INET6,
+            # ip = ip.decode('hex')
+            # return socket.inet_ntop(socket.AF_INET6,
             #          ''.join(ip[i:i+4][::-1] for i in xrange(0, 16, 4)))
             ip = base64.b16decode(ip)
             # see: http://code.google.com/p/psutil/issues/detail?id=201
             if sys.byteorder == 'little':
-                ip = socket.inet_ntop(socket.AF_INET6,
-                                struct.pack('>4I', *struct.unpack('<4I', ip)))
+                ip = socket.inet_ntop(
+                    socket.AF_INET6,
+                    struct.pack('>4I', *struct.unpack('<4I', ip)))
             else:
-                ip = socket.inet_ntop(socket.AF_INET6,
-                                struct.pack('<4I', *struct.unpack('<4I', ip)))
+                ip = socket.inet_ntop(
+                    socket.AF_INET6,
+                    struct.pack('<4I', *struct.unpack('<4I', ip)))
         return (ip, port)
