@@ -1,8 +1,6 @@
 #!/usr/bin/env python
-#
-# $Id: _psmswindows.py 1397 2012-06-29 16:33:08Z g.rodola $
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
@@ -11,25 +9,14 @@
 import errno
 import os
 import sys
-import platform
+import warnings
 
 import _psutil_mswindows
-from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
+
+from _psutil_mswindows import ERROR_ACCESS_DENIED
 from psutil._common import *
-from psutil._compat import PY3, xrange, long
-
-# Windows specific extended namespace
-__extra__all__ = ["ABOVE_NORMAL_PRIORITY_CLASS", "BELOW_NORMAL_PRIORITY_CLASS",
-                  "HIGH_PRIORITY_CLASS", "IDLE_PRIORITY_CLASS",
-                  "NORMAL_PRIORITY_CLASS", "REALTIME_PRIORITY_CLASS"]
-
-
-# --- module level constants (gets pushed up to psutil module)
-
-NUM_CPUS = _psutil_mswindows.get_num_cpus()
-BOOT_TIME = _psutil_mswindows.get_system_uptime()
-ERROR_ACCESS_DENIED = 5
-WAIT_TIMEOUT = 0x00000102 # 258 in decimal
+from psutil._compat import PY3, xrange, wraps
+from psutil._error import AccessDenied, NoSuchProcess, TimeoutExpired
 
 # process priority constants:
 # http://msdn.microsoft.com/en-us/library/ms686219(v=vs.85).aspx
@@ -41,10 +28,63 @@ from _psutil_mswindows import (ABOVE_NORMAL_PRIORITY_CLASS,
                                REALTIME_PRIORITY_CLASS,
                                INFINITE)
 
+
+# Windows specific extended namespace
+__extra__all__ = ["ABOVE_NORMAL_PRIORITY_CLASS", "BELOW_NORMAL_PRIORITY_CLASS",
+                  "HIGH_PRIORITY_CLASS", "IDLE_PRIORITY_CLASS",
+                  "NORMAL_PRIORITY_CLASS", "REALTIME_PRIORITY_CLASS",
+                  #
+                  "CONN_DELETE_TCB",
+                  ]
+
+# --- module level constants (gets pushed up to psutil module)
+
+# Since these constants get determined at import time we do not want to
+# crash immediately; instead we'll set them to None and most likely
+# we'll crash later as they're used for determining process CPU stats
+# and creation_time
+try:
+    NUM_CPUS = _psutil_mswindows.get_num_cpus()
+except Exception:
+    NUM_CPUS = None
+    warnings.warn("couldn't determine platform's NUM_CPUS", RuntimeWarning)
+try:
+    BOOT_TIME = _psutil_mswindows.get_system_boot_time()
+except Exception:
+    BOOT_TIME = None
+    warnings.warn("couldn't determine platform's BOOT_TIME", RuntimeWarning)
+try:
+    TOTAL_PHYMEM = _psutil_mswindows.get_virtual_mem()[0]
+except Exception:
+    TOTAL_PHYMEM = None
+    warnings.warn("couldn't determine platform's TOTAL_PHYMEM", RuntimeWarning)
+
+CONN_DELETE_TCB = "DELETE_TCB"
+WAIT_TIMEOUT = 0x00000102  # 258 in decimal
+ACCESS_DENIED_SET = frozenset([errno.EPERM, errno.EACCES, ERROR_ACCESS_DENIED])
+
+TCP_STATUSES = {
+    _psutil_mswindows.MIB_TCP_STATE_ESTAB: CONN_ESTABLISHED,
+    _psutil_mswindows.MIB_TCP_STATE_SYN_SENT: CONN_SYN_SENT,
+    _psutil_mswindows.MIB_TCP_STATE_SYN_RCVD: CONN_SYN_RECV,
+    _psutil_mswindows.MIB_TCP_STATE_FIN_WAIT1: CONN_FIN_WAIT1,
+    _psutil_mswindows.MIB_TCP_STATE_FIN_WAIT2: CONN_FIN_WAIT2,
+    _psutil_mswindows.MIB_TCP_STATE_TIME_WAIT: CONN_TIME_WAIT,
+    _psutil_mswindows.MIB_TCP_STATE_CLOSED: CONN_CLOSE,
+    _psutil_mswindows.MIB_TCP_STATE_CLOSE_WAIT: CONN_CLOSE_WAIT,
+    _psutil_mswindows.MIB_TCP_STATE_LAST_ACK: CONN_LAST_ACK,
+    _psutil_mswindows.MIB_TCP_STATE_LISTEN: CONN_LISTEN,
+    _psutil_mswindows.MIB_TCP_STATE_CLOSING: CONN_CLOSING,
+    _psutil_mswindows.MIB_TCP_STATE_DELETE_TCB: CONN_DELETE_TCB,
+    _psutil_mswindows.PSUTIL_CONN_NONE: CONN_NONE,
+}
+
+
 @memoize
 def _win32_QueryDosDevice(s):
     return _psutil_mswindows.win32_QueryDosDevice(s)
 
+
 def _convert_raw_path(s):
     # convert paths using native DOS format like:
     # "\Device\HarddiskVolume1\Windows\systemew\file.txt"
@@ -58,28 +98,43 @@ def _convert_raw_path(s):
 
 # --- public functions
 
-def phymem_usage():
-    """Physical system memory as a (total, used, free) tuple."""
-    all = _psutil_mswindows.get_system_phymem()
-    total, free, total_pagef, avail_pagef, total_virt, free_virt, percent = all
+get_system_boot_time = _psutil_mswindows.get_system_boot_time
+# ...so that we can test it from test_memory_leask.py
+get_num_cpus = _psutil_mswindows.get_num_cpus()
+
+
+nt_virtmem_info = namedtuple('vmem', ' '.join([
+    # all platforms
+    'total', 'available', 'percent', 'used', 'free']))
+
+def virtual_memory():
+    """System virtual memory as a namedtuple."""
+    mem = _psutil_mswindows.get_virtual_mem()
+    totphys, availphys, totpagef, availpagef, totvirt, freevirt = mem
+    #
+    total = totphys
+    avail = availphys
+    free = availphys
+    used = total - avail
+    percent = usage_percent((total - avail), total, _round=1)
+    return nt_virtmem_info(total, avail, percent, used, free)
+
+
+def swap_memory():
+    """Swap system memory as a (total, used, free, sin, sout) tuple."""
+    mem = _psutil_mswindows.get_virtual_mem()
+    total = mem[2]
+    free = mem[3]
     used = total - free
-    return nt_sysmeminfo(total, used, free, round(percent, 1))
+    percent = usage_percent(used, total, _round=1)
+    return nt_swapmeminfo(total, used, free, percent, 0, 0)
 
-def virtmem_usage():
-    """Virtual system memory as a (total, used, free) tuple."""
-    all = _psutil_mswindows.get_system_phymem()
-    total_virt = all[4]
-    free_virt = all[5]
-    used = total_virt - free_virt
-    percent = usage_percent(used, total_virt, _round=1)
-    return nt_sysmeminfo(total_virt, used, free_virt, percent)
 
 def get_disk_usage(path):
     """Return disk usage associated with path."""
     try:
         total, free = _psutil_mswindows.get_disk_usage(path)
     except WindowsError:
-        err = sys.exc_info()[1]
         if not os.path.exists(path):
             raise OSError(errno.ENOENT, "No such file or directory: '%s'" % path)
         raise
@@ -87,6 +142,7 @@ def get_disk_usage(path):
     percent = usage_percent(used, total, _round=1)
     return nt_diskinfo(total, used, free, percent)
 
+
 def disk_partitions(all):
     """Return disk partitions."""
     rawlist = _psutil_mswindows.get_disk_partitions(all)
@@ -97,23 +153,20 @@ _cputimes_ntuple = namedtuple('cputimes', 'user system idle')
 
 def get_system_cpu_times():
     """Return system CPU times as a named tuple."""
-    user, system, idle = 0, 0, 0
-    # computes system global times summing each processor value
-    for cpu_time in _psutil_mswindows.get_system_cpu_times():
-        user += cpu_time[0]
-        system += cpu_time[1]
-        idle += cpu_time[2]
+    user, system, idle = _psutil_mswindows.get_system_cpu_times()
     return _cputimes_ntuple(user, system, idle)
 
+
 def get_system_per_cpu_times():
     """Return system per-CPU times as a list of named tuples."""
     ret = []
-    for cpu_t in _psutil_mswindows.get_system_cpu_times():
+    for cpu_t in _psutil_mswindows.get_system_per_cpu_times():
         user, system, idle = cpu_t
         item = _cputimes_ntuple(user, system, idle)
         ret.append(item)
     return ret
 
+
 def get_system_users():
     """Return currently connected users as a list of namedtuples."""
     retlist = []
@@ -124,24 +177,25 @@ def get_system_users():
         retlist.append(nt)
     return retlist
 
+
 get_pid_list = _psutil_mswindows.get_pid_list
 pid_exists = _psutil_mswindows.pid_exists
-network_io_counters = _psutil_mswindows.get_network_io_counters
+net_io_counters = _psutil_mswindows.get_net_io_counters
 disk_io_counters = _psutil_mswindows.get_disk_io_counters
+get_ppid_map = _psutil_mswindows.get_ppid_map  # not meant to be public
 
-# --- decorator
 
-def wrap_exceptions(callable):
-    """Call callable into a try/except clause so that if a
-    WindowsError 5 AccessDenied exception is raised we translate it
-    into psutil.AccessDenied
+def wrap_exceptions(fun):
+    """Decorator which translates bare OSError and WindowsError
+    exceptions into NoSuchProcess and AccessDenied.
     """
+    @wraps(fun)
     def wrapper(self, *args, **kwargs):
         try:
-            return callable(self, *args, **kwargs)
+            return fun(self, *args, **kwargs)
         except OSError:
             err = sys.exc_info()[1]
-            if err.errno in (errno.EPERM, errno.EACCES, ERROR_ACCESS_DENIED):
+            if err.errno in ACCESS_DENIED_SET:
                 raise AccessDenied(self.pid, self._process_name)
             if err.errno == errno.ESRCH:
                 raise NoSuchProcess(self.pid, self._process_name)
@@ -158,41 +212,72 @@ class Process(object):
         self.pid = pid
         self._process_name = None
 
-
     @wrap_exceptions
     def get_process_name(self):
-        """Return process name as a string of limited len (15)."""
-        return _psutil_mswindows.get_process_name(self.pid)
+        """Return process name, which on Windows is always the final
+        part of the executable.
+        """
+        # This is how PIDs 0 and 4 are always represented in taskmgr
+        # and process-hacker.
+        if self.pid == 0:
+            return "System Idle Process"
+        elif self.pid == 4:
+            return "System"
+        else:
+            return os.path.basename(self.get_process_exe())
 
     @wrap_exceptions
     def get_process_exe(self):
-        exe = _convert_raw_path(_psutil_mswindows.get_process_exe(self.pid))
-        # Note: if the folder containing the executable is changed
-        # GetProcessImageFileName() returns the previous name, hence
-        # this check. An empty string signals to try to guess the exe
-        # from cmdline later.
-        if not os.path.exists(exe):
-            exe = ""
-        return exe
+        # Note: os.path.exists(path) may return False even if the file
+        # is there, see:
+        # http://stackoverflow.com/questions/3112546/os-path-exists-lies
+        return _convert_raw_path(_psutil_mswindows.get_process_exe(self.pid))
 
     @wrap_exceptions
     def get_process_cmdline(self):
         """Return process cmdline as a list of arguments."""
         return _psutil_mswindows.get_process_cmdline(self.pid)
 
-    @wrap_exceptions
     def get_process_ppid(self):
         """Return process parent pid."""
-        return _psutil_mswindows.get_process_ppid(self.pid)
+        try:
+            return get_ppid_map()[self.pid]
+        except KeyError:
+            raise NoSuchProcess(self.pid, self._process_name)
+
+    def _get_raw_meminfo(self):
+        try:
+            return _psutil_mswindows.get_process_memory_info(self.pid)
+        except OSError:
+            err = sys.exc_info()[1]
+            if err.errno in ACCESS_DENIED_SET:
+                return _psutil_mswindows.get_process_memory_info_2(self.pid)
+            raise
 
     @wrap_exceptions
     def get_memory_info(self):
         """Returns a tuple or RSS/VMS memory usage in bytes."""
-        # special case for 0 (kernel processes) PID
-        if self.pid == 0:
-            return nt_meminfo(0, 0)
-        rss, vms = _psutil_mswindows.get_memory_info(self.pid)
-        return nt_meminfo(rss, vms)
+        # on Windows RSS == WorkingSetSize and VSM == PagefileUsage
+        # fields of PROCESS_MEMORY_COUNTERS struct:
+        # http://msdn.microsoft.com/en-us/library/windows/desktop/ms684877(v=vs.85).aspx
+        t = self._get_raw_meminfo()
+        return nt_meminfo(t[2], t[7])
+
+    _nt_ext_mem = namedtuple('meminfo', ' '.join([
+        'num_page_faults',
+        'peak_wset',
+        'wset',
+        'peak_paged_pool',
+        'paged_pool',
+        'peak_nonpaged_pool',
+        'nonpaged_pool',
+        'pagefile',
+        'peak_pagefile',
+        'private']))
+
+    @wrap_exceptions
+    def get_ext_memory_info(self):
+        return self._nt_ext_mem(*self._get_raw_meminfo())
 
     nt_mmap_grouped = namedtuple('mmap', 'path rss')
     nt_mmap_ext = namedtuple('mmap', 'addr perms path rss')
@@ -244,7 +329,13 @@ class Process(object):
         # special case for kernel process PIDs; return system boot time
         if self.pid in (0, 4):
             return BOOT_TIME
-        return _psutil_mswindows.get_process_create_time(self.pid)
+        try:
+            return _psutil_mswindows.get_process_create_time(self.pid)
+        except OSError:
+            err = sys.exc_info()[1]
+            if err.errno in ACCESS_DENIED_SET:
+                return _psutil_mswindows.get_process_create_time_2(self.pid)
+            raise
 
     @wrap_exceptions
     def get_process_num_threads(self):
@@ -261,8 +352,15 @@ class Process(object):
 
     @wrap_exceptions
     def get_cpu_times(self):
-        user, system = _psutil_mswindows.get_process_cpu_times(self.pid)
-        return nt_cputimes(user, system)
+        try:
+            ret = _psutil_mswindows.get_process_cpu_times(self.pid)
+        except OSError:
+            err = sys.exc_info()[1]
+            if err.errno in ACCESS_DENIED_SET:
+                ret = _psutil_mswindows.get_process_cpu_times_2(self.pid)
+            else:
+                raise
+        return nt_cputimes(*ret)
 
     @wrap_exceptions
     def suspend_process(self):
@@ -293,7 +391,7 @@ class Process(object):
         raw_file_names = _psutil_mswindows.get_process_open_files(self.pid)
         for file in raw_file_names:
             file = _convert_raw_path(file)
-            if os.path.isfile(file) and file not in retlist:
+            if isfile_strict(file) and file not in retlist:
                 ntuple = nt_openfile(file, -1)
                 retlist.append(ntuple)
         return retlist
@@ -304,8 +402,15 @@ class Process(object):
             raise ValueError("invalid %r kind argument; choose between %s"
                              % (kind, ', '.join([repr(x) for x in conn_tmap])))
         families, types = conn_tmap[kind]
-        ret = _psutil_mswindows.get_process_connections(self.pid, families, types)
-        return [nt_connection(*conn) for conn in ret]
+        rawlist = _psutil_mswindows.get_process_connections(self.pid, families,
+                                                            types)
+        ret = []
+        for item in rawlist:
+            fd, fam, type, laddr, raddr, status = item
+            status = TCP_STATUSES[status]
+            nt = nt_connection(fd, fam, type, laddr, raddr, status)
+            ret.append(nt)
+        return ret
 
     @wrap_exceptions
     def get_process_nice(self):
@@ -315,10 +420,33 @@ class Process(object):
     def set_process_nice(self, value):
         return _psutil_mswindows.set_process_priority(self.pid, value)
 
+    # available on Windows >= Vista
+    if hasattr(_psutil_mswindows, "get_process_io_priority"):
+        @wrap_exceptions
+        def get_process_ionice(self):
+            return _psutil_mswindows.get_process_io_priority(self.pid)
+
+        @wrap_exceptions
+        def set_process_ionice(self, value, _):
+            if _:
+                raise TypeError("set_process_ionice() on Windows takes only "
+                                "1 argument (2 given)")
+            if value not in (2, 1, 0):
+                raise ValueError("value must be 2 (normal), 1 (low) or 0 "
+                                 "(very low); got %r" % value)
+            return _psutil_mswindows.set_process_io_priority(self.pid, value)
+
     @wrap_exceptions
     def get_process_io_counters(self):
-        rc, wc, rb, wb =_psutil_mswindows.get_process_io_counters(self.pid)
-        return nt_io(rc, wc, rb, wb)
+        try:
+            ret = _psutil_mswindows.get_process_io_counters(self.pid)
+        except OSError:
+            err = sys.exc_info()[1]
+            if err.errno in ACCESS_DENIED_SET:
+                ret = _psutil_mswindows.get_process_io_counters_2(self.pid)
+            else:
+                raise
+        return nt_io(*ret)
 
     @wrap_exceptions
     def get_process_status(self):
@@ -341,9 +469,7 @@ class Process(object):
                 raise ValueError("invalid argument %r" % l)
             out = 0
             for b in l:
-                if not isinstance(b, (int, long)) or b < 0:
-                    raise ValueError("invalid argument %r" % b)
-                out |= 2**b
+                out |= 2 ** b
             return out
 
         # SetProcessAffinityMask() states that ERROR_INVALID_PARAMETER
@@ -352,11 +478,22 @@ class Process(object):
         allcpus = list(range(len(get_system_per_cpu_times())))
         for cpu in value:
             if cpu not in allcpus:
-                raise ValueError("invalid CPU %i" % cpu)
+                raise ValueError("invalid CPU %r" % cpu)
 
         bitmask = to_bitmask(value)
         _psutil_mswindows.set_process_cpu_affinity(self.pid, bitmask)
 
     @wrap_exceptions
     def get_num_handles(self):
-        return _psutil_mswindows.get_process_num_handles(self.pid)
+        try:
+            return _psutil_mswindows.get_process_num_handles(self.pid)
+        except OSError:
+            err = sys.exc_info()[1]
+            if err.errno in ACCESS_DENIED_SET:
+                return _psutil_mswindows.get_process_num_handles_2(self.pid)
+            raise
+
+    @wrap_exceptions
+    def get_num_ctx_switches(self):
+        tupl = _psutil_mswindows.get_process_num_ctx_switches(self.pid)
+        return nt_ctxsw(*tupl)
