@@ -1,23 +1,25 @@
 #!/usr/bin/env python
-#
-# $Id: _osx.py 1335 2012-06-03 18:21:54Z g.rodola $
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
 """OSX specific tests.  These are implicitly run by test_psutil.py."""
 
-import unittest
-import subprocess
-import time
-import sys
 import os
+import re
+import subprocess
+import sys
+import time
+import unittest
 
 import psutil
 
 from psutil._compat import PY3
-from test_psutil import reap_children, get_test_subprocess, sh
+from test_psutil import *
+
+
+PAGESIZE = os.sysconf("SC_PAGE_SIZE")
 
 
 def sysctl(cmdline):
@@ -34,6 +36,17 @@ def sysctl(cmdline):
         return result
 
 
+def vm_stat(field):
+    """Wrapper around 'vm_stat' cmdline utility."""
+    out = sh('vm_stat')
+    for line in out.split('\n'):
+        if field in line:
+            break
+    else:
+        raise ValueError("line not found")
+    return int(re.search('\d+', line).group(0)) * PAGESIZE
+
+
 class OSXSpecificTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -42,17 +55,8 @@ class OSXSpecificTestCase(unittest.TestCase):
     def tearDown(self):
         reap_children()
 
-    def test_TOTAL_PHYMEM(self):
-        sysctl_hwphymem = sysctl('sysctl hw.memsize')
-        self.assertEqual(sysctl_hwphymem, psutil.TOTAL_PHYMEM)
-
-    def test_total_virtmem(self):
-        tot1 = psutil.virtmem_usage().total
-        tot2 = os.path.getsize("/var/vm/swapfile0")
-        self.assertEqual(tot1, tot2)
-
     def test_process_create_time(self):
-        cmdline = "ps -o lstart -p %s" %self.pid
+        cmdline = "ps -o lstart -p %s" % self.pid
         p = subprocess.Popen(cmdline, shell=1, stdout=subprocess.PIPE)
         output = p.communicate()[0]
         if PY3:
@@ -90,8 +94,64 @@ class OSXSpecificTestCase(unittest.TestCase):
             if abs(usage.used - used) > 10 * 1024 * 1024:
                 self.fail("psutil=%s, df=%s" % usage.used, used)
 
+    # --- virtual mem
 
-if __name__ == '__main__':
+    def test_vmem_total(self):
+        sysctl_hwphymem = sysctl('sysctl hw.memsize')
+        self.assertEqual(sysctl_hwphymem, psutil.TOTAL_PHYMEM)
+
+    @retry_before_failing()
+    def test_vmem_free(self):
+        num = vm_stat("free")
+        self.assertAlmostEqual(psutil.virtual_memory().free, num,
+                               delta=TOLERANCE)
+
+    @retry_before_failing()
+    def test_vmem_active(self):
+        num = vm_stat("active")
+        self.assertAlmostEqual(psutil.virtual_memory().active, num,
+                               delta=TOLERANCE)
+
+    @retry_before_failing()
+    def test_vmem_inactive(self):
+        num = vm_stat("inactive")
+        self.assertAlmostEqual(psutil.virtual_memory().inactive, num,
+                               delta=TOLERANCE)
+
+    @retry_before_failing()
+    def test_vmem_wired(self):
+        num = vm_stat("wired")
+        self.assertAlmostEqual(psutil.virtual_memory().wired, num,
+                               delta=TOLERANCE)
+
+    # --- swap mem
+
+    def test_swapmem_sin(self):
+        num = vm_stat("Pageins")
+        self.assertEqual(psutil.swap_memory().sin, num)
+
+    def test_swapmem_sout(self):
+        num = vm_stat("Pageouts")
+        self.assertEqual(psutil.swap_memory().sout, num)
+
+    def test_swapmem_total(self):
+        tot1 = psutil.swap_memory().total
+        tot2 = 0
+        # OSX uses multiple cache files:
+        # http://en.wikipedia.org/wiki/Paging#OS_X
+        for name in os.listdir("/var/vm/"):
+            file = os.path.join("/var/vm", name)
+            if os.path.isfile(file):
+                tot2 += os.path.getsize(file)
+        self.assertEqual(tot1, tot2)
+
+
+def test_main():
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(OSXSpecificTestCase))
-    unittest.TextTestRunner(verbosity=2).run(test_suite)
+    result = unittest.TextTestRunner(verbosity=2).run(test_suite)
+    return result.wasSuccessful()
+
+if __name__ == '__main__':
+    if not test_main():
+        sys.exit(1)
