@@ -1,45 +1,55 @@
 #!/usr/bin/env python
-#
-# $Id: _windows.py 1394 2012-06-29 03:21:43Z g.rodola $
-#
-# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+
+# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
 """Windows specific tests.  These are implicitly run by test_psutil.py."""
 
+import errno
 import os
-import unittest
 import platform
 import signal
-import time
-import warnings
-import atexit
-import sys
 import subprocess
-import errno
+import sys
+import time
+import traceback
+import unittest
 
-import psutil
 import _psutil_mswindows
-from psutil._compat import PY3, callable
-from test_psutil import reap_children, get_test_subprocess, wait_for_pid
+import psutil
+from psutil._compat import PY3, callable, long
+from psutil._psmswindows import ACCESS_DENIED_SET
+from test_psutil import *
+
 try:
     import wmi
 except ImportError:
     err = sys.exc_info()[1]
-    atexit.register(warnings.warn, "Couldn't run wmi tests: %s" % str(err),
-                    RuntimeWarning)
+    register_warning("Couldn't run wmi tests: %s" % str(err))
     wmi = None
 try:
     import win32api
     import win32con
 except ImportError:
     err = sys.exc_info()[1]
-    atexit.register(warnings.warn, "Couldn't run pywin32 tests: %s" % str(err),
-                    RuntimeWarning)
+    register_warning("Couldn't run pywin32 tests: %s" % str(err))
     win32api = None
 
 
+def wrap_exceptions(callable):
+    def wrapper(self, *args, **kwargs):
+        try:
+            return callable(self, *args, **kwargs)
+        except OSError:
+            err = sys.exc_info()[1]
+            if err.errno in ACCESS_DENIED_SET:
+                raise psutil.AccessDenied(None, None)
+            if err.errno == errno.ESRCH:
+                raise psutil.NoSuchProcess(None, None)
+            raise
+    return wrapper
+
 
 class WindowsSpecificTestCase(unittest.TestCase):
 
@@ -81,7 +91,7 @@ class WindowsSpecificTestCase(unittest.TestCase):
         out = p.communicate()[0]
         if PY3:
             out = str(out, sys.stdout.encoding)
-        nics = psutil.network_io_counters(pernic=True).keys()
+        nics = psutil.net_io_counters(pernic=True).keys()
         for nic in nics:
             if "pseudo-interface" in nic.replace(' ', '-').lower():
                 continue
@@ -118,7 +128,7 @@ class WindowsSpecificTestCase(unittest.TestCase):
             w = wmi.WMI().Win32_Process(ProcessId=self.pid)[0]
             p = psutil.Process(self.pid)
             domain, _, username = w.GetOwner()
-            username = "%s\\%s" %(domain, username)
+            username = "%s\\%s" % (domain, username)
             self.assertEqual(p.username, username)
 
         def test_process_rss_memory(self):
@@ -149,9 +159,9 @@ class WindowsSpecificTestCase(unittest.TestCase):
                                           time.localtime(p.create_time))
             self.assertEqual(wmic_create, psutil_create)
 
-
         # --- psutil namespace functions and constants tests
-
+        @unittest.skipUnless(hasattr(os, 'NUMBER_OF_PROCESSORS'),
+                             'NUMBER_OF_PROCESSORS env var is not available')
         def test_NUM_CPUS(self):
             num_cpus = int(os.environ['NUMBER_OF_PROCESSORS'])
             self.assertEqual(num_cpus, psutil.NUM_CPUS)
@@ -180,8 +190,9 @@ class WindowsSpecificTestCase(unittest.TestCase):
             psutil_pids = psutil.get_pid_list()
             psutil_pids.sort()
             if wmi_pids != psutil_pids:
-                difference = filter(lambda x:x not in wmi_pids, psutil_pids) + \
-                             filter(lambda x:x not in psutil_pids, wmi_pids)
+                difference = \
+                    filter(lambda x: x not in wmi_pids, psutil_pids) + \
+                    filter(lambda x: x not in psutil_pids, wmi_pids)
                 self.fail("difference: " + str(difference))
 
         def test_disks(self):
@@ -220,7 +231,7 @@ class WindowsSpecificTestCase(unittest.TestCase):
             handle = win32api.OpenProcess(win32con.PROCESS_QUERY_INFORMATION,
                                           win32con.FALSE, os.getpid())
             after = p.get_num_handles()
-            self.assertEqual(after, before+1)
+            self.assertEqual(after, before + 1)
             win32api.CloseHandle(handle)
             self.assertEqual(p.get_num_handles(), before)
 
@@ -235,13 +246,12 @@ class WindowsSpecificTestCase(unittest.TestCase):
                     ret = attr
 
             p = psutil.Process(self.pid)
-            attrs = []
             failures = []
             for name in dir(psutil.Process):
                 if name.startswith('_') \
-                or name.startswith('set_') \
-                or name in ('terminate', 'kill', 'suspend', 'resume', 'nice',
-                            'send_signal', 'wait', 'get_children'):
+                    or name.startswith('set_') \
+                    or name in ('terminate', 'kill', 'suspend', 'resume', 'nice',
+                                'send_signal', 'wait', 'get_children', 'as_dict'):
                     continue
                 else:
                     try:
@@ -260,7 +270,100 @@ class WindowsSpecificTestCase(unittest.TestCase):
                 self.fail('\n' + '\n'.join(failures))
 
 
-if __name__ == '__main__':
+class TestDualProcessImplementation(unittest.TestCase):
+    fun_names = [
+        # function name                 tolerance
+        ('get_process_cpu_times',       0.2),
+        ('get_process_create_time',     0.5),
+        ('get_process_num_handles',     1),  # 1 because impl #1 opens a handle
+        ('get_process_io_counters',     0),
+        ('get_process_memory_info',     1024),  # KB
+    ]
+
+    def test_compare_values(self):
+        # Certain APIs on Windows have 2 internal implementations, one
+        # based on documented Windows APIs, another one based
+        # NtQuerySystemInformation() which gets called as fallback in
+        # case the first fails because of limited permission error.
+        # Here we test that the two methods return the exact same value,
+        # see:
+        # http://code.google.com/p/psutil/issues/detail?id=304
+        def assert_ge_0(obj):
+            if isinstance(obj, tuple):
+                for value in obj:
+                    self.assertGreaterEqual(value, 0)
+            elif isinstance(obj, (int, long, float)):
+                self.assertGreaterEqual(obj, 0)
+            else:
+                assert 0  # case not handled which needs to be fixed
+
+        def compare_with_tolerance(ret1, ret2, tolerance):
+            if ret1 == ret2:
+                return
+            else:
+                if isinstance(ret2, (int, long, float)):
+                    diff = abs(ret1 - ret2)
+                    self.assertLessEqual(diff, tolerance)
+                elif isinstance(ret2, tuple):
+                    for a, b in zip(ret1, ret2):
+                        diff = abs(a - b)
+                        self.assertLessEqual(diff, tolerance)
+
+        failures = []
+        for name, tolerance in self.fun_names:
+            meth1 = wrap_exceptions(getattr(_psutil_mswindows, name))
+            meth2 = wrap_exceptions(getattr(_psutil_mswindows, name + '_2'))
+            for p in psutil.process_iter():
+                if name == 'get_process_memory_info' and p.pid == os.getpid():
+                    continue
+                #
+                try:
+                    ret1 = meth1(p.pid)
+                except psutil.NoSuchProcess:
+                    continue
+                except psutil.AccessDenied:
+                    ret1 = None
+                #
+                try:
+                    ret2 = meth2(p.pid)
+                except psutil.NoSuchProcess:
+                    # this is supposed to fail only in case of zombie process
+                    # never for permission error
+                    continue
+
+                # compare values
+                try:
+                    if ret1 is None:
+                        assert_ge_0(ret2)
+                    else:
+                        compare_with_tolerance(ret1, ret2, tolerance)
+                        assert_ge_0(ret1)
+                        assert_ge_0(ret2)
+                except AssertionError:
+                    trace = traceback.format_exc()
+                    msg = '%s\npid=%s, method=%r, ret_1=%r, ret_2=%r' % (
+                          trace, p.pid, name, ret1, ret2)
+                    failures.append(msg)
+                    break
+        if failures:
+            self.fail('\n\n'.join(failures))
+
+    def test_zombies(self):
+        # test that NPS is raised by the 2nd implementation in case a
+        # process no longer exists
+        ZOMBIE_PID = max(psutil.get_pid_list()) + 5000
+        for name, _ in self.fun_names:
+            meth = wrap_exceptions(getattr(_psutil_mswindows, name))
+            self.assertRaises(psutil.NoSuchProcess, meth, ZOMBIE_PID)
+
+
+def test_main():
     test_suite = unittest.TestSuite()
     test_suite.addTest(unittest.makeSuite(WindowsSpecificTestCase))
-    unittest.TextTestRunner(verbosity=2).run(test_suite)
+    test_suite.addTest(unittest.makeSuite(TestDualProcessImplementation))
+    result = unittest.TextTestRunner(verbosity=2).run(test_suite)
+    return result.wasSuccessful()
+
+if __name__ == '__main__':
+    if not test_main():
+        sys.exit(1)
